from rest_framework import serializers
from wagtail.templatetags.wagtailcore_tags import richtext
from .models import Term


class TermSerializer(serializers.ModelSerializer):
    definition = serializers.SerializerMethodField()

    class Meta:
        model = Term
        fields = ["term", "definition", "id"]

    def get_definition(self, obj):
        return richtext(obj.definition)



from rest_framework.viewsets import ReadOnlyModelViewSet

from .models import Term
from .serializers import TermSerializer

from wagtail.search.backends import get_search_backend


class TermViewSet(ReadOnlyModelViewSet):
    serializer_class = TermSerializer

    def get_queryset(self):
        q = self.request.query_params.get("q")
        queryset = Term.objects.all()
        if not self.request.user.is_staff:
            queryset = queryset.filter(live=True)
        if q:
            return get_search_backend().autocomplete(q, queryset)
        else:
            return queryset



from django.urls import path
from .views import TermViewSet

app_name = "wagtailterms"

urlpatterns = [
    path("", TermViewSet.as_view({"get": "list"}), name="terms-list"),
    path("<int:pk>/", TermViewSet.as_view({"get": "retrieve"}), name="terms-detail"),
]



from django.urls import reverse
from wagtail import hooks
from draftjs_exporter.dom import DOM
import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from wagtail.admin.panels import FieldPanel
from wagtail.admin.rich_text.converters.html_to_contentstate import (
    InlineEntityElementHandler,
)
from django.utils.safestring import mark_safe
from wagtail.snippets.models import register_snippet
from wagtail.snippets.views.snippets import SnippetViewSet
from .default_settings import get_setting
from .models import Term

TERM_ICON = get_setting('icon')


@hooks.register("insert_editor_js")
def editor_js():
    # add the path to the terms list view to the javascript so that the url can be set dynamically for the terms api
    path_to_term = reverse("wagtailterms:terms-list")
    wagtail_term_styles = get_setting('style')
    return mark_safe(f'<script>const WAGTAIL_TERM_PATH = "{path_to_term}"; const WAGTAIL_TERM_STYLE = "{wagtail_term_styles}"</script>')


@hooks.register("register_rich_text_features")
def register_term_feature(features):
    features.default_features.append("term")
    """
    Registering the `term` feature, which uses the `TERM` Draft.js entity type,
    and is stored as HTML with a `<span data-term>` tag.
    """
    feature_name = "term"
    type_ = "TERM"

    control = {
        "type": type_,
        "icon": TERM_ICON,
        "description": "Term",
    }

    features.register_editor_plugin(
        "draftail",
        feature_name,
        draftail_features.EntityFeature(
            control,
            js=[
                "wagtailterms/popup-js-1.4.2.js",
                "wagtailterms/term.js",
                "wagtailterms/popperjs.js",
                "wagtailterms/tippyjs.js",
            ],
        ),
    )

    features.register_converter_rule(
        "contentstate",
        feature_name,
        {
            # Note here that the conversion is more complicated than for blocks and inline styles.
            "from_database_format": {
                "span[data-term]": TermEntityElementHandler(type_)
            },
            "to_database_format": {"entity_decorators": {type_: term_entity_decorator}},
        },
    )


def term_entity_decorator(props):
    """
    Draft.js ContentState to database HTML.
    Converts the TERM entities into a span tag.
    """
    return DOM.create_element(
        "span",
        {
            "style": get_setting('style'),
            "data-term": props["term"]["id"],
        },
        props["children"],
    )


class TermEntityElementHandler(InlineEntityElementHandler):
    """
    Database HTML to Draft.js ContentState.
    Converts the span tag into a TERM entity, with the right data.
    """

    mutability = "MUTABLE"

    def get_attribute_data(self, attrs):
        """
        Take the `term` value from the `data-term` HTML attribute.
        """
        try:
            term = Term.objects.get(id=attrs["data-term"])
            return {
                "term": {
                    "term": term.term,
                    "definition": term.definition,
                    "id": term.id,
                }
            }
        except Term.DoesNotExist:
            return {
                "term": {
                    "term": "<span style='color:red'>Term Not Found</span>",
                    "definition": "<i>This term might be deleted</i>",
                    "id": 0,
                }
            }


class TermViewSet(SnippetViewSet):
    model = Term

    panels = [FieldPanel("term"), FieldPanel("definition")]
    icon = TERM_ICON
    add_to_admin_menu = True
    menu_label = "Terms"
    menu_name = "term"
    menu_order = get_setting('menu_order')


register_snippet(TermViewSet)



# Generated by Django 5.0.3 on 2024-03-07 02:37

import django.db.models.deletion
import wagtail.fields
import wagtail.search.index
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("wagtailcore", "0091_remove_revision_submitted_for_moderation"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Term",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "live",
                    models.BooleanField(
                        default=True, editable=False, verbose_name="live"
                    ),
                ),
                (
                    "has_unpublished_changes",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        verbose_name="has unpublished changes",
                    ),
                ),
                (
                    "first_published_at",
                    models.DateTimeField(
                        blank=True,
                        db_index=True,
                        null=True,
                        verbose_name="first published at",
                    ),
                ),
                (
                    "last_published_at",
                    models.DateTimeField(
                        editable=False, null=True, verbose_name="last published at"
                    ),
                ),
                (
                    "go_live_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="go live date/time"
                    ),
                ),
                (
                    "expire_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="expiry date/time"
                    ),
                ),
                (
                    "expired",
                    models.BooleanField(
                        default=False, editable=False, verbose_name="expired"
                    ),
                ),
                (
                    "locked",
                    models.BooleanField(
                        default=False, editable=False, verbose_name="locked"
                    ),
                ),
                (
                    "locked_at",
                    models.DateTimeField(
                        editable=False, null=True, verbose_name="locked at"
                    ),
                ),
                ("term", models.CharField(max_length=25)),
                ("definition", wagtail.fields.RichTextField()),
                (
                    "latest_revision",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.revision",
                        verbose_name="latest revision",
                    ),
                ),
                (
                    "live_revision",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.revision",
                        verbose_name="live revision",
                    ),
                ),
                (
                    "locked_by",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="locked_%(class)ss",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="locked by",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
    ]



from django.test import override_settings
from rest_framework.test import APITestCase
from django.contrib.auth import get_user_model
from .models import Term
from django.urls import reverse


class TestTermEntity(APITestCase):
    @classmethod
    def setUpTestData(cls):
        cls.admin_user = get_user_model().objects.create_superuser(
                "admin", password="pass"
        )
        cls.normal_user = get_user_model().objects.create_user("user", password="pass")
        cls.term1 = Term.objects.create(
                term="Test Term", definition="Test Definition", live=True
        )
        cls.term2 = Term.objects.create(
                term="Test Term 2", definition="Test Definition 2", live=True
        )
        cls.term3 = Term.objects.create(
                term="Test Term 3", definition="Test Definition 3", live=True
        )

        # not live terms should not be visible to non staff
        cls.term4 = Term.objects.create(
                term="Test Term 4", definition="Test Definition 4", live=False
        )
        cls.term5 = Term.objects.create(
                term="Test Term 5", definition="Test Definition 5", live=False
        )

    def test_can_view_term(self):
        """test that a term can be viewed with the rest api"""
        response = self.client.get(reverse("wagtailterms:terms-list"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 3)

        self.assertEqual(response.data[0]["term"], "Test Term")
        self.assertEqual(response.data[0]["definition"], "Test Definition")

        # try view terms when logged in as normal user
        self.client.login(username="user", password="pass")
        response = self.client.get(reverse("wagtailterms:terms-list"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 3)

    def test_admin_can_view(self):
        self.client.login(username="admin", password="pass")
        response = self.client.get(reverse("wagtailterms:terms-list"))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 5)

    def test_can_search_term(self):
        response = self.client.get(
                f"{reverse('wagtailterms:terms-list')}?q=Test Term 2"
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]["term"], "Test Term 2")

    def test_can_change_icon(self):
        from . import wagtail_hooks

        self.assertEqual(wagtail_hooks.TERM_ICON, 'snippet')

        with self.settings(WAGTAILTERMS={'icon': "bin"}):
            from . import wagtail_hooks

            self.assertEqual(wagtail_hooks.get_setting('icon'), 'bin')

    def test_menu_order_works(self):
        from . import wagtail_hooks

        self.assertEqual(wagtail_hooks.TermViewSet.menu_order, 200)

    @override_settings(WAGTAILTERMS={'menu_order': 900})
    def test_can_change_menu_order(self):
        from . import wagtail_hooks
        self.assertEqual(wagtail_hooks.get_setting('menu_order'), 900)



from django.db import models
from wagtail.fields import RichTextField
from wagtail.models import DraftStateMixin, RevisionMixin, LockableMixin
from wagtail.search import index


# Create your models here.
class Term(index.Indexed, DraftStateMixin, RevisionMixin, LockableMixin, models.Model):
    term = models.CharField(max_length=25)
    definition = RichTextField()

    search_fields = [
        index.AutocompleteField("term", partial_match=True),
        index.FilterField("live"),
    ]

    def __str__(self):
        return self.term



import os

from setuptools import setup
from os import path

install_requires = [
    "wagtail>=5.2",
    "djangorestframework>=3.12.0",
]

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
        name="wagtailterms",
        version=os.environ.get("RELEASE_VERSION", '0.0.1'),
        description="A Wagtail plugin to add support for glossary terms entity to Draftail",
        long_description=long_description,
        long_description_content_type="text/markdown",
        url="https://github.com/smark-1/wagtailterms/",
        download_url="https://pypi.python.org/pypi/wagtailterms",
        license="MIT",
        packages=["wagtailterms"],
        install_requires=install_requires,
        include_package_data=True,
        keywords=[
            "wagtail",
            "draftjs",
            "Draftail",
            "picker",
            "term",
            "definition",
            "glossary",
        ],
        classifiers=[
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Programming Language :: Python :: 3.12",
            "Framework :: Django",
            "Framework :: Wagtail",
            "Framework :: Wagtail :: 5",
            "Framework :: Wagtail :: 6",
        ],
)



from django.conf import settings

# icon
# term_style
# menu_position
DEFAULT_WAGTAILTERMS_SETTINGS = {
    'icon': 'snippet',
    'menu_order': 200,
    'style': "text-decoration-line: underline; text-decoration-color: green;text-decoration-thickness: 3px;color:green;"
}


def get_setting(setting):
    if hasattr(settings, 'WAGTAILTERMS'):
        return settings.WAGTAILTERMS.get(setting, DEFAULT_WAGTAILTERMS_SETTINGS.get(setting))
    else:
        return DEFAULT_WAGTAILTERMS_SETTINGS.get(setting)
